package com.dsic.gj_erp.aspect;

import com.alibaba.fastjson.JSONObject;
import com.dsic.gj_erp.annotation.Dict;
import com.dsic.gj_erp.bean.ResultBean;
import com.dsic.gj_erp.bean.sy.Dm_zhbmp;
import com.dsic.gj_erp.dao.sy.Dm_zhbmpRepository;
import com.dsic.gj_erp.util.ObjConvertUtils;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 数据字典切面
 *
 * @author xhj
 * @date 2021/1/18
 */
@Aspect
@Component
@Slf4j
public class DictAspect {

	/**
	* 字典后缀
	*/
    private static String DICT_TEXT_SUFFIX = "Text";

    @Autowired
    private Dm_zhbmpRepository dm_zhbmpRepository;

	/**
	* 切点，切入 controller 包下面的所有方法
     * 定义切入点表达式  execution (* com.demo.service.impl..*.*(..))
     *  1、execution(): 表达式主体。
     *  2.、方法修饰符  无
     *  3、第一个*号：表示返回类型，*号表示所有的类型。
     *  4、包名：表示需要拦截的包名，后面的两个句点表示当前包和当前包的所有子包，com.demo.service.impl包、子孙包下所有类的方法。
     *  5、第二个*号：表示类名，*号表示所有的类。
     *  6、*(..):最后这个星号表示方法名，*号表示所有的方法，后面括弧里面表示方法的参数，两个句点表示任何参数。
     *  7、异常模式 不限
	*/
    //@Pointcut("execution( * com.dsic.xzerp.controller..*.*Controller.*(..))")
      @Pointcut("execution( * com.dsic.gj_erp.controller..*Controller.*Dict(..))")

    public void dict() {

    }


    @Around("dict()")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
        long time1 = System.currentTimeMillis();
        Object result = pjp.proceed();
        long time2 = System.currentTimeMillis();
        log.info("获取JSON数据 耗时：" + (time2 - time1) + "ms");
        long start = System.currentTimeMillis();
        this.parseDictText(result);
        long end = System.currentTimeMillis();
        log.info("解析注入JSON数据  耗时" + (end - start) + "ms");
        return result;
    }
    // 一个controller 返回多个是不好用的如果是map qukey 进行遍历
    private void parseDictText(Object result) {
        log.info("进到这里来了");
        if (result instanceof ResultBean) {
            List<JSONObject> items = new ArrayList<>();
            ResultBean rr = (ResultBean) result;
            List listVo=null;
            if ( rr.getData() instanceof Map){
                listVo= (List) ((Map) rr.getData()).get("list");
            }else if (rr.getData() instanceof List){
                listVo= (List) rr.getData();
            }

            if (listVo!=null &&listVo.size()>0) {
                List<?> list = (List<?>) rr.getData();
                for (Object record : list) {
                    ObjectMapper mapper = new ObjectMapper();
                    String json = "{}";
                    try {
                        // 解决@JsonFormat注解解析不了的问题详见SysAnnouncement类的@JsonFormat
                        json = mapper.writeValueAsString(record);
                    } catch (JsonProcessingException e) {
                        log.error("Json解析失败：" + e);
                    }
                    JSONObject item = JSONObject.parseObject(json);
                    // 解决继承实体字段无法翻译问题
                    for (Field field : ObjConvertUtils.getAllFields(record)) {
                        //解决继承实体字段无法翻译问题
                        // 如果该属性上面有@Dict注解，则进行翻译
                        if (field.getAnnotation(Dict.class) != null) {
                            // 拿到注解的dictDataSource属性的值
                            String dictType = field.getAnnotation(Dict.class).dictCode();
                            // 拿到注解的dictText属性的值
                            String text = field.getAnnotation(Dict.class).dictText();
                            //获取当前带翻译的值
                            String key = String.valueOf(item.get(field.getName()));
                            //翻译字典值对应的text值
                            String textValue = translateDictValue(dictType, key);
                            // DICT_TEXT_SUFFIX的值为，是默认值：
                            // public static final String DICT_TEXT_SUFFIX = "_dictText";
                         //   log.debug("字典Val: " + textValue);
                         //   log.debug("翻译字典字段：" + field.getName() + DICT_TEXT_SUFFIX + "： " + textValue);
                            //如果给了文本名
                            if (!StringUtils.isBlank(text)) {
                                item.put(text, textValue);
                            } else {
                                // 走默认策略
                                item.put(field.getName() + DICT_TEXT_SUFFIX, textValue);
                            }
                        }
                        // date类型默认转换string格式化日期
                        if ("java.util.Date".equals(field.getType().getName())
                                && field.getAnnotation(JsonFormat.class) == null
                                && item.get(field.getName()) != null) {
                            SimpleDateFormat aDate = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                            item.put(field.getName(), aDate.format(new Date((Long) item.get(field.getName()))));
                        }
                    }
                    items.add(item);
                }
                rr.setData(items);
            }
        }
    }

    /**
     * 翻译字典文本
     *
     * @param dictType
     * @param key
     * @return
     */
    private String translateDictValue(String dictType, String key) {
        if (ObjConvertUtils.isEmpty(key)) {
            return null;
        }
        StringBuffer textValue = new StringBuffer();
        String[] keys = key.split(",");
        for (String k : keys) {
            if (k.trim().length() == 0) {
                continue;
            }
            /**
            * 根据 dictCode 和 code 查询字典值，例如：dictCode:sex,code:1，返回:男
            * 应该放在redis，提高响应速度
            */
//            SysDictDetail dictData = dm_zhbmpRepository.getDictDataByTypeAndValue(dictType, key);
            Dm_zhbmp dictData = dm_zhbmpRepository.findByBMLBAndBM(dictType,key);
            //System.out.println("执行了");
            if (dictData.getBmsm() != null) {
                if (!"".equals(textValue.toString())) {
                    textValue.append(",");
                }
                textValue.append(dictData.getBmsm());
            }
            //log.info("数据字典翻译: 字典类型：{}，当前翻译值：{}，翻译结果：{}", dictType, k.trim(), dictData.getBMSM());
        }
        return textValue.toString();
    }
}